<?php

declare(strict_types=1);

namespace App\Controller;

use ApiPlatform\Validator\ValidatorInterface;
use App\Dto\Input\DeployImageInput;
use App\Entity\Command;
use App\Entity\Image;
use App\Entity\ImageImageRepository;
use App\Entity\OrganizationalUnit;
use App\Entity\Partition;
use App\Model\CommandTypes;
use App\Model\DeployMethodTypes;
use App\Model\TraceStatus;
use App\Service\Trace\CreateService;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Attribute\Route;
use Symfony\Contracts\HttpClient\Exception\ClientExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\RedirectionExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\ServerExceptionInterface;
use Symfony\Contracts\HttpClient\HttpClientInterface;

class DeployImageAction extends AbstractController
{
    public function __construct(
        protected readonly EntityManagerInterface $entityManager,
        protected readonly HttpClientInterface    $httpClient,
        protected readonly CreateService          $createService,
        protected readonly ValidatorInterface  $validator,
        public readonly \App\Controller\OgAgent\DeployImageAction $deployImageOgAgentAction,
        public readonly \App\Controller\OgRepository\Image\DeployImageAction $deployImageOgRepositoryAction,
    )
    {
    }

    /**
     * @throws RedirectionExceptionInterface
     * @throws ClientExceptionInterface
     * @throws ServerExceptionInterface
     */
    public function __invoke(DeployImageInput $input, ImageImageRepository $image): JsonResponse
    {
        $this->validator->validate($input);

        switch ($input->method){
            case DeployMethodTypes::UNICAST:
            case DeployMethodTypes::UNICAST_DIRECT:
                foreach ($input->clients as $client) {
                    $inputData = [
                        'method' => $input->method,
                        'client' => $client->getEntity()->getUuid(),
                        'image' => $image->getUuid(),
                        'numDisk' => (string) $input->diskNumber,
                        'numPartition' => (string) $input->partitionNumber,
                    ];

                    $agentJobId = $this->deployImageOgAgentAction->__invoke($image, $input, $client->getEntity(), DeployMethodTypes::UNICAST);
                    if (!$agentJobId){
                        continue;
                    }

                    $this->createService->__invoke($client->getEntity(), CommandTypes::DEPLOY_IMAGE, TraceStatus::IN_PROGRESS, $agentJobId, $inputData);
                }
            break;


            case DeployMethodTypes::MULTICAST_UFTP:
            case DeployMethodTypes::MULTICAST_UFTP_DIRECT:
            case DeployMethodTypes::MULTICAST_UDPCAST:
            case DeployMethodTypes::MULTICAST_UDPCAST_DIRECT:
                foreach ($input->clients as $client) {
                    $inputData = [
                        'method' => $input->method,
                        'client' => $client->getEntity()->getUuid(),
                        'image' => $image->getUuid(),
                        'mcastIp' => $input->mcastIp,
                        'mcastPort' => $input->mcastPort,
                        'mcastSpeed' => $input->mcastSpeed,
                        'mcastMode' => $input->mcastMode,
                        'numDisk' => (string) $input->diskNumber,
                        'numPartition' => (string) $input->partitionNumber,
                    ];

                    try {
                        $this->deployImageOgRepositoryAction->__invoke($input, $image, $client->getEntity());
                    } catch (\Exception $e) {
                        continue;
                    }

                    $agentJobId = $this->deployImageOgAgentAction->__invoke($image, $input, $client->getEntity(), DeployMethodTypes::MULTICAST);
                    if (!$agentJobId){
                        continue;
                    }

                    $this->createService->__invoke($client->getEntity(), CommandTypes::DEPLOY_IMAGE, TraceStatus::IN_PROGRESS, $agentJobId, $inputData);
                }
            break;

            case DeployMethodTypes::TORRENT:
                foreach ($input->clients as $client) {
                    $inputData = [
                        'method' => $input->method,
                        'client' => $client->getEntity()->getUuid(),
                        'image' => $image->getUuid(),
                        'p2pMode' => $input->p2pMode,
                        'p2pTime' => $input->p2pTime,
                        'numDisk' => (string) $input->diskNumber,
                        'numPartition' => (string) $input->partitionNumber,
                    ];

                    try {
                        $this->deployImageOgRepositoryAction->__invoke($input, $image, $client->getEntity(), $this->httpClient);
                    } catch (\Exception $e) {
                        continue;
                    }

                    $agentJobId = $this->deployImageOgAgentAction->__invoke($image, $input, $client->getEntity(), DeployMethodTypes::TORRENT);
                    if (!$agentJobId){
                        continue;
                    }

                    $this->createService->__invoke($client->getEntity(), CommandTypes::DEPLOY_IMAGE, TraceStatus::IN_PROGRESS, $agentJobId, $inputData);
                }

                break;
        }

        return new JsonResponse(data: [], status: Response::HTTP_OK);
    }
}
